import logging
import os
import os.path as osp
import shutil

import zendron

# set logger
log = logging.getLogger(__name__)


from omegaconf import OmegaConf


def load_default_config():
    workspace_root = os.getcwd()
    default_config_path = osp.join(workspace_root, "conf", "default.yaml")
    if not osp.exists(default_config_path):
        return False
    return OmegaConf.load(default_config_path)


def is_initialized() -> bool:
    try:
        is_zendron_initialized = load_default_config().initialized
    except:
        is_zendron_initialized = False
    return is_zendron_initialized


def set_initialized():
    default_config = load_default_config()
    default_config.initialized = True
    workspace_root = os.getcwd()
    default_config_path = osp.join(workspace_root, "conf", "default.yaml")
    OmegaConf.save(default_config, default_config_path)

    # Update config.import.yml with desired values
    config_path = osp.join(
        workspace_root, "pods", "dendron.markdown", "config.import.yml"
    )
    config = OmegaConf.load(config_path)
    config.vaultName = workspace_root.split("/")[-1]
    OmegaConf.save(config, config_path)


def user_configure() -> None:
    if not is_initialized():
        package_root = osp.dirname(osp.abspath(zendron.__file__))
        # package_root = "/".join(package_root.split("/")[:-2])
        # List all the config files and their paths within the package
        config_files = [
            "conf/__init__.py",
            "conf/default_template.yaml",
            "conf/config_template.yaml",
            "pods/dendron.markdown/config.import.yml",
        ]

        # Copy each config file to the user's workspace root directory
        for file_path in config_files:
            src_path = osp.join(package_root, file_path)
            dest_base_path, file_ext = osp.splitext(file_path)
            dest_base_path = dest_base_path.replace("_template", "")
            dest_path = osp.join(os.getcwd(), dest_base_path + file_ext)

            # Ensure the destination folder exists
            dest_folder = osp.dirname(dest_path)
            os.makedirs(dest_folder, exist_ok=True)

            if not osp.exists(dest_path):
                shutil.copy2(src_path, dest_path)
                print(f"{file_path} has been copied to the workspace root directory.")
            else:
                print(f"{file_path} already exists in the workspace root directory.")
        set_initialized()
        print("Initialization complete.")
        print(
            "Manually set the STARTER CONFIG in './conf/config.yaml' for quickstart and rerun the zendron. All other configs are optional."
        )
        return True
    else:
        print("Zendron has already been initialized.")
        return False


if __name__ == "__main__":
    user_configure()



# %%
import glob
import json
import logging
import os
import os.path as osp
from dataclasses import dataclass, field
from datetime import datetime

import frontmatter
import hydra
from html2text import html2text
from hydra import compose, initialize
from markdown import markdown
from markdownify import markdownify as md
from omegaconf import DictConfig, OmegaConf
from pyzotero import zotero
from tqdm import tqdm

from zendron import front
from zendron.annotations import AnnotationsCompiler
from zendron.metadata import Metadata, MetadataCompiler


@dataclass
class Comment:
    zot: zotero.Zotero
    attachments: list
    _note: str = None
    _comment_key: str = None
    _version: int = None
    _parentItem: str = None

    @property
    def comment_key(self) -> str:
        if self._comment_key is None:
            try:
                self._comment_key = [
                    i["key"]
                    for i in self.attachments
                    if i["zendron_title"] == "zendron comment"
                ][0]
            except IndexError:
                self._comment_key = None
        return self._comment_key

    @property
    def version(self) -> int:
        if self._version is None:
            self._version = self.zot.item(self.comment_key)["data"]["version"]
        return self._version

    @property
    def parentItem(self) -> str:
        if self._parentItem is None:
            try:
                self._parentItem = self.zot.item(self.comment_key)["data"]["parentItem"]
            except AttributeError:
                self._parentItem = ""
        return self._parentItem

    @property
    def note(self) -> str:
        if self._note is None:
            try:
                self._note = self.zot.item(self.comment_key)["data"]["note"]
            except AttributeError:
                self._note = ""
        return self._note


class CommentCompiler:
    def __init__(
        self,
        comment: Comment = None,
        title_dendron: str = None,
        dendron_limb: str = None,
    ):
        self.comment = comment
        self.title_dendron = title_dendron
        self.dendron_limb: str = dendron_limb
        self.project_dir: str = os.getcwd().split("/")[-1]
        self.line: str = None

    def compile(self, comment: Comment = None):
        self.line = comment.note
        return self.line

    def write_comment(self, pod_path: str = "zotero_pod"):
        path = [pod_path]
        path.extend(self.dendron_limb.split("."))
        path.append(self.title_dendron)
        path = "/".join(path)
        os.makedirs(path, exist_ok=True)
        file_path = osp.join(path, "comments.md")
        with open(file_path, "w") as f:
            f.write(self.line)
            f.write("\n")
        front.add_comment_key(self.comment.comment_key, file_path)


@hydra.main(
    version_base=None, config_path=osp.join(os.getcwd(), "conf"), config_name="config"
)
def main(cfg: DictConfig):
    api_key = cfg.api_key
    library_id = cfg.library_id
    library_type = cfg.library_type  # or 'user'
    dendron_limb = cfg.dendron_limb
    zot = zotero.Zotero(library_id, library_type, api_key)

    # comment = Comment()
    # comment_compiler = CommentCompiler()
    # comment_compiler.compile()
    # comment_compiler.write_comment()


if __name__ == "__main__":
    main()



# __init__.py

from importlib import resources

try:
    import tomllib
except ModuleNotFoundError:
    import tomli as tomllib

# Version of the zendron package
__version__ = "1.0.27"

# Read URL of the Real Python feed from config file
# _cfg = tomllib.loads(resources.read_text("reader", "config.toml"))
# URL = _cfg["feed"]["url"]
URL = "https://github.com/Mjvolk3/Zendron"



import json
import logging
import os
import os.path as osp
import shutil
from dataclasses import dataclass, field
from datetime import datetime

from omegaconf import DictConfig, OmegaConf
from pyzotero import zotero
from tqdm import tqdm

from zendron.date import utc_parse

log = logging.getLogger(__name__)


@dataclass
class Annotation:
    annotation: dict = None
    local_image_path: str = None
    name: str = field(init=False, repr=False)

    def __post_init__(self):
        self.name: str = self.annotation["library"]["name"]

    def get_color(self):
        color = self.annotation["data"]["annotationColor"]
        color_emoji = ""
        if color == "#5fb236":
            color_emoji = "🟢 "
        elif color == "#2ea8e5":
            color_emoji = "🔵 "
        elif color == "#a28ae5":
            color_emoji = "🟣 "
        elif color == "#ff6666":
            color_emoji = "🔴 "
        elif color == "#ffd400":
            color_emoji = "🟡 "
        else:
            color_emoji = "❓ "
        return color_emoji

    def get_annotation_text(self):
        try:
            annotation_text = self.annotation["data"]["annotationText"]
        except KeyError:
            annotation_text = ""
        return annotation_text

    def get_annotation_author_name(self):
        try:
            annotation_author_name = self.annotation["meta"]["createdByUser"][
                "username"
            ]
            # previously works for user libraries
            # annotation_author_name = self.annotation["data"]["annotationAuthorName"]
            # TODO need to test if this works in user libraries
        except KeyError:
            annotation_author_name = "Unknown"
        return annotation_author_name

    def get_annotation_comment(self):
        annotation_comment = self.annotation["data"]["annotationComment"]
        return annotation_comment

    def format_annotation_comment(
        self,
        lines: list = None,
        annotation_comment: str = "",
        color_emoji: str = "",
        color_emoji_cfg: bool = True,
    ):
        if annotation_comment != "":
            if color_emoji_cfg is True:
                color_indicator = f"({color_emoji}) "
            else:
                color_indicator = ""
            lines.append(f"- Comment {color_indicator}: {annotation_comment}")
        else:
            if color_emoji_cfg is True:
                color_indicator = f"({color_emoji}) "
            else:
                color_indicator = ""
            # For some reason there is an extra space added. Reason for [:-1].
            lines.append(f"- Comment {color_indicator}"[:-1])

        return lines

    def get_annotation_tags(self):
        annotation_tags = ", ".join(
            [f"#{i['tag']}" for i in self.annotation["data"]["tags"]]
        )
        return annotation_tags

    def format_annotation_tags(self, lines: list, annotation_tags: str):
        cfg_annotations_tags_format = True
        if annotation_tags != "":
            if cfg_annotations_tags_format:
                lines.append(f"- Tags: {annotation_tags}")
        return lines

    def format_annotation_text(self, lines: list, annotation_text: str):
        if annotation_text == "":
            return lines
        lines.append(f"\n> {annotation_text}")
        return lines

    def get_annotation_link(self):
        pass

    def get_date_added(self):
        date_added = utc_parse(self.annotation["data"]["dateAdded"])
        return date_added

    def get_image(self):
        # HACK for images
        if self.annotation["data"]["annotationType"] != "image":
            return None
        image_id = self.annotation["data"]["key"]
        if self.annotation["library"]["type"] == "user":
            image_src = f"library/{image_id}.png"
        elif self.annotation["library"]["type"] == "group":
            image_src = f"groups/{self.annotation['library']['id']}/{image_id}.png"
        image_src = osp.join(self.local_image_path, image_src)
        image_dest = f"notes/assets/images/zendron-image-import-{image_id}.png"
        shutil.copyfile(image_src, image_dest)
        image_link = f"![](./{'/'.join(image_dest.split('/')[1:])})"
        return image_link


# TODO Should really take metadata as an argument. Specifically for user_path
class AnnotationsCompiler:
    def __init__(
        self,
        annotation_title: str = None,
        title_dendron: str = None,
        local_image_path: str = None,
        dendron_limb: str = None,
        pod_path: str = None,
    ):
        self.annotation_title = annotation_title
        self.title_dendron = title_dendron
        self.local_image_path = local_image_path
        self.dendron_limb: str = dendron_limb
        self.pod_path: str = pod_path
        self.project_dir: str = os.getcwd().split("/")[-1]
        self.lines: list = None

    def get_local_annotation_link(self):
        local_annotations_link = f"dendron://{self.project_dir}/{self.dendron_limb}.{self.title_dendron}.comments"
        local_annotations_link_wiki = f"[[Local Comments|{local_annotations_link}]]"
        return local_annotations_link_wiki

    # TODO check if annotations are list[dict]
    def compile(self, annotations: list = None) -> list:
        lines = []
        lines.append("## Annotations")
        # save annotations to txt file
        # TODO default sort order is ['data']['dateModified']. Add config to allow for
        for i, annot in enumerate(annotations):
            annot = Annotation(annot, self.local_image_path)
            # TODO separate out all of these into their own functions
            # annotation_keys.append(i['key'])
            # lines.append(annot['data']['annotationPageLabel'])
            date_added = annot.get_date_added()
            lines.append(f"\n### Date Added: {date_added}")
            annotation_text = annot.get_annotation_text()
            lines = annot.format_annotation_text(lines, annotation_text)
            lines.append("")
            image_link = annot.get_image()
            if image_link is not None:
                lines.append(f"{image_link}\n")
            lines.append(f"- Annotator: @{annot.get_annotation_author_name()}")
            color_emoji = annot.get_color()
            annotation_comment = annot.get_annotation_comment()
            lines = annot.format_annotation_comment(
                lines, annotation_comment, color_emoji, color_emoji_cfg=True
            )
            tags = annot.get_annotation_tags()
            lines = annot.format_annotation_tags(lines, tags)
        lines.append("\n## Local Comments\n")
        lines.append(f"- {self.get_local_annotation_link()}")
        self.lines = lines
        return lines

    def write(self):
        path = [self.pod_path]
        path.extend(self.dendron_limb.split("."))
        path.append(self.title_dendron)
        path = "/".join(path)
        os.makedirs(path, exist_ok=True)
        with open(osp.join(path, f"{self.annotation_title}.md"), "w") as f:
            for line in self.lines:
                f.write(line)
                f.write("\n")


def main():
    pass


if __name__ == "__main__":
    main()



import glob
import logging
import os
import os.path as osp
import re
import shutil
import subprocess

import hydra
from hydra import compose, initialize
from omegaconf import DictConfig, OmegaConf

log = logging.getLogger(__name__)


def remove_files_glob(pattern: str, exclude_pattern: str = None) -> None:
    log.info(f"Removing: {pattern}")
    for matched_path in glob.glob(pattern):
        if exclude_pattern and re.search(exclude_pattern, matched_path):
            continue

        if osp.isfile(matched_path):
            try:
                os.remove(matched_path)
                print(f"File '{matched_path}' has been removed.")
            except OSError as e:
                print(f"Error: {e.filename} - {e.strerror}.")
        elif osp.isdir(matched_path):
            try:
                shutil.rmtree(matched_path)
                print(f"Directory '{matched_path}' and its contents have been removed.")
            except OSError as e:
                print(f"Error: {e.filename} - {e.strerror}.")


def main(cfg: DictConfig):
    # TODO add notes config for
    log.info("Removing all Zendron Note Files")
    remove_files_glob(f"notes/{cfg.dendron_limb}.*.md", r"\.comments\.md$")
    remove_files_glob(f"notes/{cfg.dendron_limb}.date.*.md")
    remove_files_glob(f"notes/{cfg.dendron_limb}.item-type.*.md")
    remove_files_glob(f"notes/{cfg.dendron_limb}.title.*.md")
    remove_files_glob(f"notes/{cfg.dendron_limb}.authors.*.md")
    remove_files_glob("notes/user.*.md")
    remove_files_glob("notes/tags.*.md")
    remove_files_glob("notes/assets/images/zendron-image-import-*.png")
    log.info("Removing all Zendron Cache")
    remove_files_glob("zendron_cache/metadata_cache.json")
    remove_files_glob("zendron_cache/annotations_cache.json")
    log.info("Creating Missing Linked Notes with Dendron Doctor")
    subprocess.run("dendron doctor --action createMissingLinkedNotes", shell=True)
    log.info("Note Removal Complete")


if __name__ == "__main__":
    main()



import json
import logging
import os
import os.path as osp
from curses import meta
from dataclasses import dataclass, field
from datetime import datetime

import hydra
from hydra import compose, initialize
from omegaconf import DictConfig, OmegaConf
from pyzotero import zotero
from tqdm import tqdm

from zendron import front
from zendron.date import utc_parse
from zendron.items import get_annotated_attachments, get_attachments

# set logger
log = logging.getLogger(__name__)


@dataclass
class Metadata:
    metadata: dict = None
    attachments: dict = None
    dendron_limb: str = None
    project_dir: str = field(default=os.getcwd().split("/")[-1])
    title_dendron: str = field(init=False, repr=False)
    doi: str = field(init=False, repr=False)
    library_id: str = field(init=False, repr=False)
    key: str = field(init=False, repr=False)
    name: str = field(init=False, repr=False)
    _attachment_keys: str = field(default=None, repr=False)
    citation_key: str = field(init=False, repr=False)

    def __post_init__(self):
        self.title_dendron: str = self.get_title_dendron()
        self.library_id: str = self.metadata["library"]["id"]
        self.key: str = self.metadata["key"]
        self.name: str = self.metadata["library"]["name"]

    def get_title_dendron(self):
        title = self.metadata["data"]["title"]
        title = title.replace(":", "-")  # HACK should belong in a formatter
        title_dendron = "-".join(title.split(" "))
        return title_dendron

    def get_title(self):
        title = self.metadata["data"]["title"]
        link = f"dendron://{self.project_dir}/{self.dendron_limb}.title.{self.title_dendron}"
        title_wiki = f"[[{title}|{link}]]"
        return title_wiki

    def get_creators(self):
        creators = self.metadata["data"]["creators"]
        # creatorType key exists too... think it is always author... or mostly
        creators_wiki = []
        for creator in creators:
            name = []
            first_name = creator["firstName"]
            last_name = creator["lastName"]
            name.extend([first_name, last_name])
            name_alias = " ".join(name)
            name_den = "-".join(name).replace(" ", "-").replace(".", "")
            link = (
                f"dendron://{self.project_dir}/{self.dendron_limb}.authors.{name_den}"
            )
            wiki = f"[[{name_alias}|{link}]]"
            creators_wiki.append(wiki)
        creators_wiki = ", ".join(creators_wiki)
        return creators_wiki

    def mdy_to_ymd(self, date):
        month_abbr = [
            "Jan",
            "Feb",
            "Mar",
            "Apr",
            "May",
            "Jun",
            "Jul",
            "Aug",
            "Sep",
            "Oct",
            "Nov",
            "Dec",
        ]
        if (date.split(" ")[0] in month_abbr) and ("," in date.split(" ")[1]):
            # change myd to myd date format
            date = date.split(" ")
            date[1] = date[1].replace(",", "")
            date = "-".join(date)
            date = datetime.strptime(date, "%b-%d-%Y")
            date = date.strftime("%Y-%m-%d")
        return date

    def get_date(self):
        # TODO helper function - same get_access_date
        try:
            date = utc_parse(self.metadata["data"]["date"])
        except ValueError as e:
            date = self.mdy_to_ymd(self.metadata["data"]["date"])
        date = str(date).split(" ")[0]
        date_dendron = date.replace("-", ".")
        link = f"dendron://{self.project_dir}/{self.dendron_limb}.date.{date_dendron}"
        date_wiki = f"[[{date}|{link}]]"
        return date_wiki

    def get_access_date(self):
        try:
            access_date = utc_parse(self.metadata["data"]["accessDate"])
            access_date = str(access_date).split(" ")[0]
            date_dendron = access_date.replace("-", ".")
            link = (
                f"dendron://{self.project_dir}/{self.dendron_limb}.date.{date_dendron}"
            )
            access_date_wiki = f"[[{access_date}|{link}]]"
        except:
            access_date_wiki = "No access date"
        return access_date_wiki

    def get_date_added(self):
        # TODO helper function - same ad get_date_modified
        date_added = utc_parse(self.metadata["data"]["dateAdded"])
        date_added = str(date_added).split(" ")
        date_added = "-".join(date_added)
        date_added = date_added.split("+")[:-1]
        date_added = "-".join(date_added)
        date_added = date_added.replace(":", "-")
        date_dendron = date_added.replace("-", ".")
        date_dendron = date_added.replace("-", ".")
        link = f"dendron://{self.project_dir}/{self.dendron_limb}.date.{date_dendron}"
        date_added_wiki = f"[[{date_added}|{link}]]"
        return date_added_wiki

    def get_date_modified(self):
        date_modified = utc_parse(self.metadata["data"]["dateModified"])
        date_modified = str(date_modified).split(" ")
        date_modified = "-".join(date_modified)
        date_modified = date_modified.split("+")[:-1]
        date_modified = "-".join(date_modified)
        date_modified = date_modified.replace(":", "-")
        date_dendron = date_modified.replace("-", ".")
        date_dendron = date_modified.replace("-", ".")
        link = f"dendron://{self.project_dir}/{self.dendron_limb}.date.{date_dendron}"
        date_modified_wiki = f"[[{date_modified}|{link}]]"
        return date_modified_wiki

    def get_url(self):
        url = self.metadata["data"]["url"]
        if url == "":
            return "No URL"
        url = str(f"[{url}]({url})")
        return url

    def get_DOI(self):
        try:
            doi = self.metadata["data"]["DOI"]
            self.doi = doi
            link = f"http://doi.org/{doi}"
            doi = str(f"[{doi}]({link})")
        except KeyError:
            self.doi = "No DOI"
            doi = "No DOI"
        return doi

    def get_citations(self):
        try:
            extras = self.metadata["data"]["extra"].split("\n")
            citations = [i for i in extras if "citations" in i]
            citations = citations[0].split("citations: ")[-1]
        except IndexError:
            citations = "No citations"
        return citations

    def get_pinned_citation_key(self):
        try:
            extras = self.metadata["data"]["extra"].split("\n")
            citation_key = [i for i in extras if "Citation Key" in i]
            self.citation_key = citation_key[0].split("Citation Key: ")[-1]
            citation_key_wiki = f"[[{self.citation_key}|user.{self.citation_key}]]"
        except Exception as e:
            # Not sure which error is going to be triggered here
            log.warning(
                f"No citation key for {self.title_dendron}, Check that citation keys are being pinned in Zotero."
            )
        return citation_key_wiki

    def get_publication_title(self):
        try:
            publication_title = self.metadata["data"]["publicationTitle"]
            publication_title = publication_title.replace("[", "").replace(
                "]", ""
            )  # HACK needs to go into formatter
            publication_title_dendron = "-".join(publication_title.split(" "))
            link = f"dendron://{self.project_dir}/{self.dendron_limb}.publication-title.{publication_title_dendron}"
            publication_title_wiki = f"[[{publication_title}|{link}]]"
        except KeyError:
            publication_title_wiki = "No publication title"
        return publication_title_wiki

    def get_journal_abbreviation(self):
        journal_abbreviation = self.metadata["data"]["journalAbbreviation"]
        journal_abbreviation_den = "-".join(journal_abbreviation.split(" "))
        link = f"dendron://{self.project_dir}/{self.dendron_limb}.publication-title.{journal_abbreviation_den}"
        journal_abbreviation_wiki = f"[[{journal_abbreviation}|{link}]]"
        return journal_abbreviation_wiki

    def get_item_type(self):
        item_type = self.metadata["data"]["itemType"]
        item_type_den = "-".join(item_type.split(" "))
        link = f"dendron://{self.project_dir}/{self.dendron_limb}.item-type.{item_type_den}"
        item_type_wiki = f"[[{item_type}|{link}]]"
        return item_type_wiki

    @property
    def attachment_keys(self):
        if self._attachment_keys is None:
            self._attachment_keys = [i["key"] for i in self.attachments]
        return self._attachment_keys

    def get_pdf_attachments(self):
        # TODO for groups only - need another for local. parameterize type of library.
        # TODO deal with multiple attachments... like supplementary.
        library_type = self.metadata["library"]["type"]
        if library_type == "user":
            for att_key in self.attachment_keys:
                try:
                    link = f"https://www.zotero.org/{self.name}/items/{self.key}/attachment/{att_key}/reader"
                    pdf_attachment = f"[Online PDF attachment]({link})"
                except KeyError:
                    pdf_attachment = None
                    log.warning(
                        "No PDF attachment... Your metadata should probably have attachments."
                    )
        if library_type == "group":
            for att_key in self.attachment_keys:
                try:
                    # TODO check if this works for groups
                    link = f"https://www.zotero.org/groups/{self.library_id}/zendron/items/{self.key}/attachment/{att_key}/reader"
                    pdf_attachment = f"[Online PDF attachment]({link})"
                except KeyError:
                    pdf_attachment = None
                    log.warning(
                        "No PDF attachment... Your metadata should probably have attachments."
                    )
        return pdf_attachment

    def get_tags(self):
        if self.metadata["data"]["tags"] == []:
            return "No tags"
        for tags in self.metadata["data"]["tags"]:
            if " " in tags["tag"]:
                log.warning(
                    f"Tags should not contain spaces. Please remove spaces in Zotero and sync again. Tag: {tags['tag']}. Title Dendron: {self.title_dendron}."
                )
        tags = ", ".join([f"#{i['tag']}" for i in self.metadata["data"]["tags"]])
        # TODO Config to convert tags that aren't delimited by '-' to be... On backward sync they can be updated with this. Could be a dangerous operation
        return tags

    def get_local_library(self):
        local_library = f"zotero://select/items/{self.library_id}"
        local_library_wiki = f"[Local Library]({local_library})"
        return local_library_wiki

    def get_cloud_library(self):
        cloud_library = (
            f"https://www.zotero.org/groups/{self.library_id}/{self.name}/library"
        )
        cloud_library_wiki = f"[Cloud Library]({cloud_library})"
        return cloud_library_wiki

    def get_abstract(self):
        abstract = self.metadata["data"]["abstractNote"]
        if abstract == "":
            abstract = "No abstract"
        return abstract


class MetadataCompiler:
    def __init__(self, metadata: Metadata = None, pod_path: str = None):
        self.metadata = metadata
        self.pod_path = pod_path
        self.lines: list = None

    def compile(self) -> str:
        lines = []
        # TODO implement config for adding different data.
        lines.append("## Metadata\n")
        lines.append(f"- Title: {self.metadata.get_title()}")
        lines.append(f"- Authors: {self.metadata.get_creators()}")
        lines.append(f"- Date: {self.metadata.get_date()}")
        lines.append(f"- Date Accessed: {self.metadata.get_access_date()}")
        lines.append(f"- Date Added: {self.metadata.get_date_added()}")
        lines.append(f"- Date Modified: {self.metadata.get_date_modified()}")
        lines.append(f"- URL: {self.metadata.get_url()}")
        lines.append(f"- DOI: {self.metadata.get_DOI()}")
        lines.append(f"- Citation Key: {self.metadata.get_pinned_citation_key()}")
        lines.append(f"- Citations: {self.metadata.get_citations()}")
        lines.append(f"- Publication Title: {self.metadata.get_publication_title()}")
        lines.append(f"- Journal Abbreviation: {self.metadata.get_publication_title()}")
        lines.append(f"- Item Type: {self.metadata.get_item_type()}")
        lines.append(f"- PDF Attachments: {self.metadata.get_pdf_attachments()}")
        lines.append(f"- Tags: {self.metadata.get_tags()}")
        lines.append(f"- Local Library: {self.metadata.get_local_library()}")
        lines.append(f"- Cloud Library: {self.metadata.get_cloud_library()}")
        lines.append("\n## Abstract\n")
        lines.append(self.metadata.get_abstract())
        self.lines = lines
        return lines

    def write(self):
        path = [self.pod_path]
        path.extend(self.metadata.dendron_limb.split("."))
        path = "/".join(path)
        os.makedirs(path, exist_ok=True)
        file_path = osp.join(path, f"{self.metadata.title_dendron}.md")
        with open(file_path, "w") as f:
            for line in self.lines:
                f.write(line)
                f.write("\n")
        front.add_metadata_key(self.metadata.key, file_path)


def main():
    pass


if __name__ == "__main__":
    main()



import json
import logging
import os
import os.path as osp
import shutil
from collections import OrderedDict

import hydra
from hydra import compose, initialize
from omegaconf import DictConfig, OmegaConf
from pyzotero import zotero
from tqdm import tqdm

from zendron.annotations import Annotation, AnnotationsCompiler
from zendron.cache import Cache
from zendron.items import get_annotated_attachments, get_attachments, get_metadatas
from zendron.metadata import Metadata, MetadataCompiler

log = logging.getLogger(__name__)


# TODO when running sync make sure comments are always synced first... Probably go backwards in order of tree... annotations to metadata.
def remove_from_meta_queue(
    cache: Cache, metadata_list: list, dendron_limb: str, pod_path: str
):
    # Delete metadata that is not in the metadata_list.. These are items that have been remove from Zotero
    meta_remove_queue = list(
        set(cache.data.keys()) - set(i["key"] for i in metadata_list)
    )
    for meta_key in meta_remove_queue:
        dendron_limb_path = dendron_limb.replace(".", "/")
        delete_dir = osp.join(
            pod_path,
            dendron_limb_path,
            cache.data[meta_key]["title_dendron"],
        )
        delete_file_path = osp.join(
            pod_path,
            dendron_limb_path,
            cache.data[meta_key]["title_dendron"] + ".md",
        )
        user_file_path = osp.join(
            pod_path,
            dendron_limb_path,
            "user",
            cache.data[meta_key]["citation_key"] + ".md",
        )
        if osp.exists(delete_dir):
            log.info(f"Removing {delete_dir}")
            shutil.rmtree(delete_dir)
        if osp.exists(delete_file_path):
            log.info(f"Removing {delete_file_path}")
            os.remove(delete_file_path)
        if osp.exists(user_file_path):
            log.info(f"Removing {user_file_path}")
            os.remove(user_file_path)
        cache.delete_metadata(meta_key)


def sync_metadata(
    zot,
    metadata_list: list = None,
    dendron_limb: str = None,
    pod_path: str = None,
):
    cache = Cache(zot)
    cache.load()

    remove_from_meta_queue(cache, metadata_list, dendron_limb, pod_path)

    # Update metadata that is out versioned in cache
    meta_update_queue = []
    for meta in metadata_list:
        try:
            if meta["version"] > cache.data[meta["key"]]["version"]:
                meta_update_queue.append(meta)
        except KeyError:
            log.info(f"{meta['key']} not in cache")
            meta_update_queue.append(meta)

    for metadata in tqdm(meta_update_queue):
        attachments = get_attachments(zot, metadata["key"])
        metadata = Metadata(
            metadata,
            attachments,
            dendron_limb,
        )
        meta_compiler = MetadataCompiler(metadata, pod_path)
        meta_compiler.compile()
        meta_compiler.write()
        cache.update_metadata(metadata)
        print("")


# def sync_annotated_attachments(
#     zot,
#     all_annotated_attachments: list = None,
#     dendron_limb: str = None,
#     local_image_path: str = None,
#     pod_path: str = None,
# ):

#     cache = Cache(zot)
#     cache.load()

#     cached_annotated_attachments_temp = [
#         cache.data[k]["annotated_attachments"] for k, v in cache.data.items()
#     ]
#     cached_annotated_attachments = {}
#     for d in cached_annotated_attachments_temp:
#         cached_annotated_attachments.update(d)

#     for attach in all_annotated_attachments:
#         # for cache_att in cached_attachments
#         if (
#             attach["key"] in cached_annotated_attachments.keys()
#             and attach["version"]
#             > cached_annotated_attachments[attach["key"]]["version"]
#         ):
#             attach_update = attach
#         if attach["key"] not in cached_annotated_attachments.keys():
#             attach_update = attach

#         cache.update_annotated_attachment(attach_update)
#     print("")


def sync_annotations(
    zot,
    all_annotated_attachments: list = None,
    metadata_list: list = None,
    dendron_limb: str = None,
    local_image_path: str = None,
    pod_path: str = None,
):
    cache = Cache(zot)
    cache.load()

    annotations = []
    meta_keys = []
    for attach in all_annotated_attachments:
        annotations.extend(zot.children(attach["key"]))
        meta_keys.append(zot.item(attach["key"])["data"]["parentItem"])

    annot_queue = []
    meta_queue = []
    for annot, meta_key in zip(annotations, meta_keys):
        try:
            if annot["version"] > cache[annot["key"]]:
                annot_queue.append(annot)
                meta_queue.append(meta_key)
        except KeyError:
            log.info(f"{annot['key']} not in cache")
            annot_queue.append(annot)
            meta_queue.append(meta_key)

    # Rewriting the md file
    for annot, meta_key in zip(annot_queue, meta_queue):
        # IDEA this coupling to attachments is annoying. It might be better idea to give metadata access to zot.
        metadata = [i for i in metadata_list if i["key"] == meta_key][0]
        attachments = get_attachments(zot, meta_key)
        metadata = Metadata(
            metadata,
            attachments,
            dendron_limb,
        )
        annotated_attachments = get_annotated_attachments(zot, metadata.key)
        attach = [
            i for i in annotated_attachments if annot["data"]["parentItem"] == i["key"]
        ][0]
        # IDEA coupling to met metadata through metadata title is annoying. If Annotations had access to zot, it could get the metadata title, then title_dendron.
        annot_compiler = AnnotationsCompiler(
            attach["zendron_title"],
            metadata.title_dendron,
            local_image_path,
            dendron_limb,
            pod_path=pod_path,
        )
        annot_compiler.compile(zot.children(attach["key"]))
        annot_compiler.write()
        cache.update_annotation(annot["key"], annot["version"])


@hydra.main(
    version_base=None, config_path=osp.join(os.getcwd(), "conf"), config_name="config"
)
def main(cfg: DictConfig):
    api_key = cfg.api_key
    library_id = cfg.library_id
    library_type = cfg.library_type
    zot = zotero.Zotero(library_id, library_type, api_key)
    metadata_list = get_metadatas(zot, cfg.collection, cfg.item_types)
    all_annotated_attachments = get_annotated_attachments(
        zot, [i["key"] for i in metadata_list]
    )
    log.info("Syncing metadata")
    sync_metadata(zot, metadata_list, cfg.dendron_limb, cfg.pod_path)
    sync_annotated_attachments(zot, all_annotated_attachments)
    # log.info("Syncing annotations")
    # sync_annotations(
    #     zot,
    #     all_annotated_attachments,
    #     metadata_list,
    #     cfg.dendron_limb,
    #     cfg.local_image_path,
    #     cfg.pod_path,
    # )
    print("")


if __name__ == "__main__":
    main()



import argparse
import logging
import os
import os.path as osp
import subprocess

import hydra
from omegaconf import DictConfig

from zendron import init, load

log = logging.getLogger(__name__)

global USER_CONFIGURE
USER_CONFIGURE = False

if init.user_configure():
    USER_CONFIGURE = True


@hydra.main(
    version_base=None, config_path=osp.join(os.getcwd(), "conf"), config_name="config"
)
def main(cfg: DictConfig) -> None:
    if USER_CONFIGURE:
        return
    elif cfg.remove:
        from zendron import remove

        log.info("Remove Starting")

        remove.main(cfg)
    else:
        log.info("Sync Starting")
        load.main(cfg)
        subprocess.run(
            "dendron importPod --podId dendron.markdown --wsRoot .", shell=True
        )
        subprocess.run("rm -r zotero_pod", shell=True)
        log.info("Sync Complete")


if __name__ == "__main__":
    main()



from markdown import markdown

with open("notes/zendron.import.variational-graph-auto-encoders.comments.md", "r") as f:
    text = f.read()
    html = markdown(text)
    print("end")

# # Update the attachment
# zot.attachment_simple(
#     ["notes/zendron.import.variational-graph-auto-encoders.comments.md"], "ITY7GKUZ"
# )
# with open("result.json", "w") as fp:
#     json.dump(zot.children("UIVEMQEC"), fp, indent=4)

# with open("result.json") as f:
#     data = json.load(f)
# print("end")

# zot.attachment_simple(["result.json"], "ITY7GKUZ")

# zot.upload_attachments(
#     data,
#     "ITY7GKUZ",
#     "notes",
# )


# with open("Picnic.html", "w") as f:
#     f.write(html)



import frontmatter


def add_metadata_key(metadata_key: str, file_path: str):
    data = frontmatter.load(file_path)
    data["metadata_key"] = metadata_key
    updated_data = frontmatter.dumps(data)
    with open(file_path, "w") as f:
        f.write(updated_data)


def add_comment_key(comment_key: str, file_path: str):
    data = frontmatter.load(file_path)
    data["comment_key"] = comment_key
    updated_data = frontmatter.dumps(data)
    with open(file_path, "w") as f:
        f.write(updated_data)


def main():
    pass


if __name__ == "__main__":
    main()



from dataclasses import dataclass

import hydra
from hydra.core.config_store import ConfigStore


@dataclass
class ZendronConfig:
    library_id : 4932032 #9025336 my user
    library_type : group
    api_key : FoCeuOvWMnNdYmpYuY5JpETw
    collection: null # null if not specified.
    item_types: [journalArticle, book, preprint, conferencePaper, report]
    local_image_path: /Users/michaelvolk/Zotero/cache
    dendron_limb: zendron.import
    zotero_comment_title: zendron comment
    pod_path: zotero_pod

def main():
    pass


if __name__ == "__main__":
    main()
